class Schema:
    __yes_vals: list[str] = ["true", "yes", "on"]

    __unique: str = "unique"
    __spec: str = "spec"
    __facets: str = "facets"
    __fields: str = "fields"
    __type: str = "type"
    __label: str = "label"

    def __init__(self, *, schema: dict) -> None:
        self.__schema = schema

    def facets(self) -> list[str]:
        return list(self.__schema[Schema.__facets])

    def fields(self) -> list[str]:
        return list(self.__schema[Schema.__fields].keys())

    def types(self) -> dict:
        return {
            field: a[Schema.__type]
            for field, a in self.__schema[Schema.__fields].items()
        }

    def is_unique(self, field: str) -> bool:
        return (
            Schema.__yes_vals in self.__schema[field][Schema.__unique].lower()
            if Schema.__unique in self.__schema[field].keys()
            else True
        )

    def needs_spec(self, field: str) -> bool:
        return (
            Schema.__yes_vals in self.__schema[field][Schema.__spec].lower()
            if Schema.__spec in self.__schema[field].keys()
            else False
        )

    # def get_value(self, meta: dict, field: str, lang: str):
    #     """
    #     treats receiving the value like a variant,
    #     returns values with their language specific representations
    #     """
    #     match self.__schema[field]["type"]:
    #         case "str":
    #             return (
    #                 meta[field][lang]
    #                 if self.is_translatable(field)
    #                 else meta[field]["value"]
    #             )
    #         case "enum" | "int" | "num" | "multikey":
    #             return meta[field]["value"]
    #         case "multinum":
    #             return (
    #                 meta[field]["value"]
    #                 if hasattr(meta[field]["value"], "__iter__")
    #                 else [
    #                     meta[field]["value"],
    #                 ]
    #             )  # force list!

    # def to_list_of_dict(self, meta, fields, lang):
    #     """
    #     generates a list of dict which can easily be converted
    #     to a pandas dataframe
    #     """
    #     # list comprehension for rows
    #     return [
    #         {
    #             "field": field,  # field name
    #             "lang": lang,  # language shortcode
    #             "type": self.__schema[field]["type"],  # datatype
    #             "label": self.__schema[field]["label"][lang],  # label
    #             "value": self.get_value(meta, field, lang),  # actual value
    #             "template": self.__schema[field]["template"][lang]
    #             if "template" in self.__schema[field]
    #             else None,
    #             # getting crazy with nested dict comprehension
    #             "enum_values": {
    #                 k: v[lang] for (k, v) in self.__schema[field]["values"].items()
    #             }
    #             if "enum" in self.__schema[field]["type"]
    #             else None,
    #             "key_values": {
    #                 k: v[lang] for (k, v) in self.__schema[field]["keys"].items()
    #             }
    #             if "multikey" in self.__schema[field]["type"]
    #             else None,
    #             "spec": meta[field]["spec"][lang] if "spec" in meta[field] else None,
    #         }
    #         for field in fields
    #     ]

    # def to_short_dict(self, meta, fields, lang):
    #     """
    #     generates a short version of dict which can easily be converted
    #     to a pandas dataframe
    #     """
    #     # dict comprehension for whole meta part
    #     return {field: self.get_value(meta, field, lang) for field in fields}

    # def to_list_of_tuple(self, meta, fields, lang):
    #     """
    #     generates a list of tuples with a label and value (text)
    #     this is usually consumed by a Markdown generator

    #     todo: needs deuglyfication of free standing loop, templates are possible for all
    #     """
    #     list = []
    #     for r in self.to_list_of_dict(meta, fields, lang):
    #         match r["type"]:
    #             case "str":
    #                 list.append((r["label"], r["value"]))
    #             case "int" | "num":
    #                 list.append(
    #                     (
    #                         r["label"],
    #                         r["template"].format(value=r["value"], spec=r["spec"])
    #                         if r["template"]
    #                         else r["value"],
    #                     )
    #                 )
    #             case "enum":
    #                 list.append(
    #                     (
    #                         r["label"],
    #                         r["template"].format(
    #                             value=r["enum_values"][r["value"]], spec=r["spec"]
    #                         )
    #                         if r["template"]
    #                         else r["enum_values"][r["value"]],
    #                     )
    #                 )
    #             case "multikey":
    #                 list.append(
    #                     (
    #                         r["label"],
    #                         ", ".join(
    #                             [
    #                                 r["template"].format(
    #                                     key=r["key_values"][k], value=v
    #                                 )
    #                                 for k, v in r["value"].items()
    #                             ]
    #                         ),
    #                     )
    #                 )
    #             case "multinum":
    #                 list.append(
    #                     (
    #                         r["label"],
    #                         ", ".join(
    #                             r["template"].format(value=v) for v in r["value"]
    #                         ),
    #                     )
    #                 )

    #     return list
